################################################################################
#
# EVENT HANDLING PROCEDURES
# -------------------------
# 
################################################################################



proc campaign_combo_changed {} {
	global campaign_combo_cur_sel
	# clear the wildcards if not 'All' is selected as campaign
    if {[.campaign_combo getvalue] != 0} {
		.camp_wildcards delete 0 end
    }
    
	fill_species_combo ;# must update the species combo
	build_sql_query ;# rebuild query
}

proc species_combo_changed {} {
	global species_combo_cur_sel
    if {[.species_combo getvalue] != 0} {
		.folder_wildcards delete 0 end
    }
   	
	# set global variable to avoid some malfunction of the combobox
	# which would not return index of current selection when called too often in a row
    set species_combo_cur_sel [.species_combo getvalue]
	
	build_sql_query ;# rebuild query
}

proc report_level_changed {} {
	build_sql_query ;# rebuild query
}

proc date_changed {} {
	global db_handle
	
	set from_date ""
	set to_date ""
	
	# get min and max of the years
	set query "select min(YEAR(capture_date)) from site"
	set min_YYYY [mysqlsel $db_handle $query -flatlist]
	set query "select max(YEAR(capture_date)) from site"
	set max_YYYY [mysqlsel $db_handle $query -flatlist]
	
	# build from_date
	set DD [.from_DD get]
	set MM [.from_MM get]
	set YYYY [.from_YYYY get]
	
	# if all of the entry fields of from_date are empty, clear the result field
	if {[str_concat $YYYY $MM $DD] eq ""} {
		.from_date configure -text ""
	} else  {
		if {$DD eq ""} {set DD "00"} else {two_char_number DD}
		if {$MM eq ""} {set MM "00"} else {two_char_number MM}
		if {$YYYY < $min_YYYY || $YYYY eq ""} {set YYYY $min_YYYY}
		if {$YYYY > $max_YYYY} {set YYYY $max_YYYY}
		
		set from_date [str_concat $YYYY - $MM - $DD]
	}
	
	.from_date configure -text $from_date
	
	# build to_date
	set DD [.to_DD get]
	set MM [.to_MM get]
	set YYYY [.to_YYYY get]
	
	# if all of the entry fields of from_date are empty, clear the result field
	if {[str_concat $YYYY $MM $DD] eq ""} {
		.to_date configure -text ""
	} else  {
		if {$DD eq ""} {set DD "00"} else {two_char_number DD}
		if {$MM eq ""} {set MM "00"} else {two_char_number MM}
		if {$YYYY < $min_YYYY} {set YYYY $max_YYYY}
		if {$YYYY > $max_YYYY || $YYYY eq ""} {set YYYY $max_YYYY}
		
		set to_date [str_concat $YYYY - $MM - $DD]
	}
	
	.to_date configure -text $to_date	
	
	build_sql_query ;# rebuild query
}

proc camp_wildcards_changed {} {
	global campaign campaign_wildcards
	
	set campaign_wildcards [.camp_wildcards get]
	
	# some string has been entered
	# now we switch the study name selection box to "All" (first entry)
	# but only if there is a string, otherwise this was called by
	# the automatic deletion when selecting a study different to 'All'
	if {$campaign_wildcards ne ""} {
        .campaign_combo setvalue first
    }
    
	build_sql_query ;# rebuild query
	fill_species_combo ;# must update the species combo
	
	return 1
}

proc spec_wildcards_changed {} {
	global species_wildcards species_combo_cur_sel
	
	set species_wildcards [.folder_wildcards get]
	
	# some string has been entered
	# now we switch the species name selection box to "All" (first entry)
	# but only if there is a string, otherwise this was called by
	# the automatic deletion when selecting a species different to 'All'
	if {$species_wildcards ne ""} {
        .species_combo setvalue first
	} 
    
    build_sql_query
	
	return 1
}

proc altitude_changed {} {
	set min_alt [.min_alt get]
	set max_alt [.min_alt get]
	
	#only accept integer input, other stuff will be deleted
	if {[string is integer $min_alt] != 1} {
		.min_alt delete 0 end
		set min_alt ""
	}
	if {[string is integer $max_alt] != 1} {
		.max_alt delete 0 end
		set max_alt ""
	}	

	build_sql_query

}

proc exec_query {} {
    global report_level
    create_query_result $report_level
}

proc reset {} {
	global report_level CAMPAIGN
	
	# report level
	set report_level $CAMPAIGN
	
	# campaign
    .campaign_combo setvalue first
	.camp_wildcards delete 0 end
	
	# species
    .species_combo setvalue first
	.folder_wildcards delete 0 end
	
	# collection dates
	.from_DD delete 0 end
	.from_MM delete 0 end
	.from_YYYY delete 0 end
	.to_DD delete 0 end
	.to_MM delete 0 end
	.to_YYYY delete 0 end
	.from_date configure -text ""
	.to_date configure -text ""
	
	# altitude
	.min_alt delete 0 end
	.max_alt delete 0 end
		
	build_sql_query
	
}


proc closeMainWindow {w} {
	global db_handle
	# Close DB connection
	mysqlclose $db_handle
	destroy $w
}


