################################################################################
#
# Selection Logic
#
################################################################################




campaign_class campaign "study"
species_class species "species"
site_class site "site"
spectrum_class spectrum "spectrum"

set table_instances [list campaign species site spectrum]


proc build_sql_query {} {
    global sql_select_query sql_count_query report_level 
    global campaign species site spectrum ;# class instances
    global CAMPAIGN SPECIES SITE SPECTRUM
    
    # reset current query
    set sql_select_query ""
    set sql_count_query ""
    
    # prepare statement fragments
    
    
    campaign get_conditions
    
    set conditions [campaign get_conditions ]
    conc_cond conditions [species get_conditions]
    conc_cond conditions [site get_conditions]
    
    # get the primary and foreign key combinations for all involved tables
    conc_cond conditions [get_key_combinations [get_involved_tables "AS_LIST"]]
    
    #.debug_txt insert end $conditions
    
    set tables [get_involved_tables]
    #.debug_txt insert end $tables
    set primary_key [get_primary_key_column_name [get_current_main_table]]
    set primary_key [prefix_with_tablename [get_current_main_table] $primary_key]
    set sel_attributes [str_concat "distinct " $primary_key]
    
    set cnt_attributes [str_concat "COUNT(distinct " $primary_key ")"]

    
    # assemble queries
    set sql_select_query [assemble_sql_query $sel_attributes $tables $conditions]
    set sql_count_query [assemble_sql_query $cnt_attributes $tables $conditions]
    
    # update query info in GUI
    update_query_info
    
}


proc get_involved_tables {{return_type "AS_STRING"}} {
    global campaign species site spectrum ;# class instances
    global table_instances
    
    set tables ""
    
    foreach table $table_instances {
        if {[$table involved]} {
            lappend tables [$table get_table_name]
        }
    }
    
    if {$return_type eq "AS_STRING"} {
        return [join $tables ", "]
    } else  {
        return $tables
    }    
}

# return conditions for the key combinations
proc get_key_combinations {tables} {
    # get key combinations (primary / FK) for all tables with each other
    # algorithm: get keys for first table in list with all others
    # then for the second table the keys with the tables later in the list, etc
    # Thus it can be called recursively!
    
    set conds ""
    
    if {[llength $tables] <= 1} {
        return $conds ;# no conditions if there is only one table (or even none!)        
    }
    
    set table1 [lindex $tables 0]
    
    foreach table2 [lrange $tables 1 end] {
        set columns [get_foreign_and_primary_keys $table1 $table2]
        conc_cond conds [str_concat [lindex $columns 0] "=" [lindex $columns 1]]
    }
    
    # recursive call: add the conditions of the resulting conditions to the existing
    conc_cond conds [get_key_combinations [lrange $tables 1 end]]
    
    return $conds
    
}

proc get_current_main_table {} {
    global report_level
    return [get_table_name_of_level $report_level]
}

proc get_table_name_of_level {level} {
    global campaign species site spectrum ;# class instances
    switch $level {
        1 { ;# "Campaign"
            return [campaign get_table_name]
        }
        2 { ;# "Species"
            return [species get_table_name]
        }
        3 { ;# "Site"
            return [site get_table_name]
        }
        4 { ;# "Spectrum"
            return [spectrum get_table_name]
        }
    }
}

# Returns 1 if the given level needs a subquery
# In general, a subquery is needed if the supplied level is not equal to the
# current main table (i.e. table associated with the current report_level)
# A return value of 1 does not mean that conditions for that table exist
proc subquery_needed {table} {
    global report_level
    global CAMPAIGN SPECIES SITE SPECTRUM
    
    return 0;# no subqueries due to speed problems
    
    if {$table != $report_level} {
        return 1
    }
    return 0
}

proc get_conditions {table} {
    global CAMPAIGN SPECIES SITE SPECTRUM
    set t_conds ""
    set conds ""
    
    # decide if a subquery is needed for this table
    set subq [subquery_needed $table] ;# 0 (false) or 1 (true)
    
    switch $table {
        1 { ;# "Campaign"
            set t_conds [get_campaign_conditions]
        }
        2 { ;# "Species"
            set t_conds [get_species_conditions]
        }
        3 { ;# "Site"
            set t_conds [get_site_conditions]
        }
        4 { ;# "Spectrum"
            set t_conds [get_spectrum_conditions]
        }
    } ;# end switch $table
    
    # only proceed if there are conditions, otherwise return the emtpy string
    if {$t_conds ne ""} {
        switch $subq {
            0 { ;# no subquery
                return $t_conds
            }
            1 { ;# subquery
                set referencing_table_name [get_current_main_table] ;# the main table name
                set referenced_table_name [get_table_name $table] ;# current table name
                
                set columns [get_foreign_and_primary_keys $referencing_table_name $referenced_table_name]
                #.debug_txt insert end $columns
                
                set cond [str_concat [lindex $columns 0] " in ("]
                
                set attr [lindex $columns 1]
                append cond [assemble_sql_query $attr $referenced_table_name $t_conds]
                append cond ")"
                return $cond
            }
        }
    }
    return ""
}




proc update_query_info {} {
    global sql_select_query sql_count_query db_handle startup
    
    # do not try display stuff during building of the GUI
    if {$startup == 0} {
        # clear text widget
        .sql_query_txt delete "1.0" end
        
        # display in text widget
        #.sql_query_txt insert end $sql_select_query
        #.sql_query_txt insert end "\n"
        .sql_query_txt insert end $sql_count_query
        
        # count and set the number of resulting rows
        .row_cnt_number configure -text [mysqlsel $db_handle $sql_count_query -flatlist]
    }
}




proc fill_species_combo {} {
    global db_handle
    global campaign species site spectrum ;# class instances
    set tables ""
    set cond ""
    set options ""
    set species_listbox [.species_combo getlistbox]
    
    # remove entries from combobox
    # set no_of_items [.species_combo list size]
    # if {$no_of_items > 0} {
    # .species_combo delete list 0 end ;#$no_of_items
    # }
    #
    $species_listbox delete 0 end
    
    # get study conditions
    set s_cond [campaign get_conditions]
    
    if {$s_cond eq ""} {
        set tables "species"
    } else  {
        set tables "species, study"
        set cond "species.study_id = study.study_id and "
        append cond $s_cond
    }
    
    set options "order by folder_name"
    
    # prepare query
    set query [assemble_sql_query "distinct folder_name" $tables $cond $options]
    
    #.debug_txt insert end $query
    
    # add first entry into combobox
    $species_listbox insert end "All"
    
    # get species from db according to current study settings
    foreach res [mysqlsel $db_handle $query -flatlist] {
        $species_listbox insert end $res
    }
    
    # set combo list to first entry
    .species_combo setvalue first
}

